/*----------------------------------------------------------------------
  File    : coconad.h
  Contents: COntinuous time ClOsed Neuron Assembly Detection
  Author  : Christian Borgelt
  History : 2013.02.22 file created
            2014.07.24 definition of CCN_CMRONE added
            2014.08.28 functions ccn_data() and ccn_report() added
            2014.12.16 interval list intersection variant added
            2015.09.04 definition of CCN_SIMSUPP added
            2016.03.24 support mixing parameter added (CCN_OVERLAP)
            2016.10.23 moved repository selection to operation mode
            2016.11.16 coconad miner object and interface introduced
----------------------------------------------------------------------*/
#ifndef __COCONAD__
#define __COCONAD__
#include "train.h"
#include "report.h"

/*----------------------------------------------------------------------
  Preprocessor Definitions
----------------------------------------------------------------------*/
/* --- target pattern types --- */
#define CCN_TYPEMASK  ~CCN_SUBMASK /* target pattern type mask */
#define CCN_SUBMASK   0x00ff    /* target pattern subtype mask */

#define CCN_ITEMSET   0x0000    /* item set (order is ignored) */
#define CCN_PERMUT    0x0100    /* partial permutation (no reps.) */
#define CCN_SEQUENCE  0x0200    /* sequence (repetitions possible) */

#define CCN_FREQ      ISR_FREQUENT  /* frequent patterns */
#define CCN_FREQUENT  ISR_FREQUENT  /* frequent patterns */
#define CCN_CLOSED    ISR_CLOSED    /* closed frequent patterns */
#define CCN_MAXIMAL   ISR_MAXIMAL   /* maximal frequent patterns */
#define CCN_APPEND    ISR_GENERAS   /* only w.r.t. appending items */

/* --- data preparation modes --- */
#define CCN_NORECODE  0x0001    /* do not sort and recode items */
#define CCN_SURR      CCN_NORECODE

/* --- algorithm variants --- */
#define CCN_BASIC     0         /* basic: linear memory consumption */
#define CCN_FILTER    1         /* refined: with spike filtering */
#define CCN_CMREPO    2         /* closed/maximal with repository */
#define CCN_REPO      2         /* closed/maximal with repository */
#ifdef CCN_OVERLAP              /* if influence map overlap support */
#define CCN_ISECT     3         /* interval list intersection */
#endif
#define CCN_AUTO      4         /* automatic choice */

/* --- operation modes --- */
#define CCN_PERFECT   0x0020    /* perfect extension pruning */
#define CCN_REORDER   0x0040    /* reorder items in cond. databases */
#ifdef CCN_OVERLAP
#define CCN_SIMSUPP   0x0080    /* exg. roles of support / similarity */
#endif
#define CCN_PREFMT    0x4000    /* pre-format integer numbers */
#define CCN_DEFAULT   (CCN_PERFECT|CCN_REORDER|CCN_PREFMT)

#define CCN_REPOCHAIN 0x0100    /* repository is a chain of trees */
#define CCN_REPOTREE  0x0200    /* repository is a single tree */
#define CCN_REPOLIST  0x0400    /* repository is a list/an array */
#define CCN_REPOTYPE  0x0700    /* mask for repository type */

#ifdef NDEBUG
#define CCN_NOCLEAN   0x8000    /* do not clean up memory */
#else                           /* in function coconad() */
#define CCN_NOCLEAN   0         /* in debug version */
#endif                          /* always clean up memory */
#define CCN_VERBOSE   INT_MIN   /* verbose message output */

/* --- similarity measures --- */
#define CCN_NONE             0  /* always 0 */
#define CCN_RUSSEL_RAO       1  /* S_R = s/n */
#define CCN_KULCZYNSKI       2  /* S_K = s/q */
#define CCN_JACCARD          3  /* S_J = s/r */
#define CCN_TANIMOTO         3
#define CCN_DICE             4  /* S_D = 2s/(r+s) */
#define CCN_SORENSEN         4
#define CCN_CZEKANOWKSI      4
#define CCN_SOKAL_SNEATH_1   5  /* S_S = s/(r+q) */
#define CCN_SOKAL_MICHENER   6  /* S_M = (s+z)/n */
#define CCN_HAMMING          6
#define CCN_FAITH            7  /* S_F = (s+z/2)/n */
#define CCN_ROGERS_TANIMOTO  8  /* S_T = (s+z)/(n+q) */
#define CCN_SOKAL_SNEATH_2   9  /* S_N = 2(s+z)/(n+s+z) */
#define CCN_GOWER_LEGENDRE   9
#define CCN_SOKAL_SNEATH_3  10  /* S_O = (s+z)/q */
#define CCN_BARONI_BUSER    11  /* S_B = (x+s)/(x+r) */
#define CCN_GENERIC         12  /* S   = (c_0s +c_1z +c_2n +c_3x)
                                       / (c_4s +c_5z +c_6n +c_7x) */

/*----------------------------------------------------------------------
  Type Definitions
----------------------------------------------------------------------*/
typedef struct _coconad         /* coconad miner */
COCONAD;                        /* (opaque structure) */

/*----------------------------------------------------------------------
  Functions
----------------------------------------------------------------------*/
extern COCONAD* ccn_create (int target, double width, double smin,
#ifdef CCN_OVERLAP              /* if influence map overlap support */
                            double mix,
#endif                          /* add. support mixing parameter */
                            ITEM zmin, ITEM zmax, int algo, int mode);
extern void     ccn_delete (COCONAD *coconad, int deldar);
extern int      ccn_data   (COCONAD *coconad, TRAINSET *trnset,
                            int mode);
extern int      ccn_report (COCONAD *coconad, ISREPORT *report);
extern int      ccn_mine   (COCONAD *coconad);
#ifdef CCN_OVERLAP
extern int      ccn_jim    (COCONAD *coconad, int simcov, double *simps,
                            double sim, double beg, double end);
#endif
#endif
